unit Editor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, Weltraumspiel, Spin, Grids;

type
  TLevelEditor = class(TForm)
    MainMenu1: TMainMenu;
    Datei: TMenuItem;
    Laden: TMenuItem;
    Speichern: TMenuItem;
    N1: TMenuItem;
    Beenden: TMenuItem;
    LevelBearbeiten: TMenuItem;
    LevelLoeschen: TMenuItem;
    LevelZufall: TMenuItem;
    DateiLesen: TOpenDialog;
    SaveLevel: TSaveDialog;
    DateiNeu: TMenuItem;
    Eingabe: TDrawGrid;
    LabelLevel: TLabel;
    SpinEditLevel: TSpinEdit;
    procedure LadenClick(Sender: TObject);
    procedure LevelErzeugen(Raum:Byte);
    procedure LevelSchreiben(Raum:Byte);
    procedure BeendenClick(Sender: TObject);
    procedure SpeichernClick(Sender: TObject);
    procedure DateiNeuClick(Sender: TObject);
    procedure EingabeDrawCell(Sender: TObject; ACol, ARow: Integer;
                              Rect: TRect; State: TGridDrawState);
    procedure SpinEditLevelChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EingabeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LevelLoeschenClick(Sender: TObject);
    procedure LevelZufallClick(Sender: TObject);
    procedure Such(Tiefe,Anzahl,x,y:Integer);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private-Deklarationen }
   BildDaten:DatenArray;
   Level:LevelRec;
   LevelNr:Integer;
   LevelFertig:Boolean;
  public
    { Public-Deklarationen }
  end;

var
  LevelEditor: TLevelEditor;

implementation

{$R *.dfm}
{$R+,Q+}
 procedure TLevelEditor.LevelErzeugen(Raum:Byte);     // Level entschlsseln
  const Wert:Array[0..7]of Byte=(128, 64, 32, 16, 8, 4, 2, 1);
  var x,y:Integer;
  Begin
   Level.x:=BildDaten.Level[Raum].x;                  // Startposition
   Level.y:=BildDaten.Level[Raum].y;
   For x := -2 to 9 do
    For y := -2 to 9 do Level.Feld[x,y]:=255;           // Rahmen
   For x := 0 to 7 do
    For y := 0 to 7 do
     if (BildDaten.Level[Raum].Zeile[x] AND Wert[y])>0 then Level.Feld[x,y]:=1
     else Level.Feld[x,y]:=0;
  End;

 procedure TLevelEditor.LevelSchreiben(Raum:Byte);    // Level verschlsseln
  const Wert:Array[0..7]of Byte=(128, 64, 32, 16, 8, 4, 2, 1);
  var x,y:Integer;
  Begin
   For x:=0 to 7 do
    Begin
     BildDaten.Level[Raum].Zeile[x]:=0;
     For y := 0 to 7 do
      if Level.Feld[x,y]=1 then
       BildDaten.Level[Raum].Zeile[x]:=BildDaten.Level[Raum].Zeile[x] OR Wert[y];
    End;
   BildDaten.Level[Raum].x:=Level.x;
   BildDaten.Level[Raum].y:=Level.y;
  End;

 procedure TLevelEditor.LadenClick(Sender: TObject);
  const LevelKennung:String[13]='Weltraumspiel';
  var BildDatei:File of DatenArray;
  begin
   with DateiLesen do
    begin
     Filter:='Weltraumdateien (*.lev) |*.lev';
     DefaultExt:='lev';
     if Execute then
      begin
       AssignFile(BildDatei,FileName);{$I-}Reset(BildDatei);{$I+}
       IF IOResult<>0 THEN
        BEGIN
         MessageDlg('Fehler bei Dateizugriff', mtWarning, [mbOk], 0);
         MessageDlg(FileName+' kann ich nicht ffnen!',mtError,[mbOk],0);
         Exit;
        END;
       {$I-} IF FileSize(BildDatei)=1 THEN Read(BildDatei,BildDaten);{$I+}
       IF (BildDaten.Kennung<>LevelKennung) OR (IOResult<>0) THEN
        BEGIN
         MessageDlg(FileName+' ist keine Leveldatei fr Weltraumspiel!',mtError,[mbOk],0);
         MessageDlg('Das Programm wird beendet...',mtConfirmation,[mbOk],0);
         Halt;
        END;
       CloseFile(BildDatei);
       LevelNr:=1;LevelErzeugen(LevelNr);
       SpinEditLevel.Value:=1;
       SpinEditLevel.MaxValue:=99;
       Eingabe.Repaint;
      end
    end
  end;

 procedure TLevelEditor.SpeichernClick(Sender: TObject);
  const LevelKennung:String[13]='Weltraumspiel';
  var BildDatei:File of DatenArray;
  begin
   LevelSchreiben(LevelNr);
   with SaveLevel do
    begin
     Filter:='Weltraumdateien (*.lev) |*.lev';
     DefaultExt:='lev';
     Options:=[ofOverwritePrompt]; // Sicherheitsabfrage wegen berschreibens
     if Execute then
      begin
       BildDaten.Kennung:=LevelKennung;
       AssignFile(BildDatei,FileName);{$I-}Rewrite(BildDatei);{$I+}
       IF IOResult<>0 THEN
        BEGIN
         MessageDlg('Die Level-Datei '+FileName+' kann ich nicht speichern!',mtError,[mbOk],0);
         Exit;
        END;
       {$I-}Write(BildDatei,BildDaten);{$I+}
       IF IOResult<>0 THEN
        BEGIN
         MessageDlg('Schreibfehler durch '+FileName,mtError,[mbOk],0);
         Exit;
        END;
       CloseFile(BildDatei);
      end
    end
  end;

 procedure TLevelEditor.DateiNeuClick(Sender: TObject);
  const LeerRaum:DatenRec=(Zeile:(0,0,0,0,0,0,0,0); x:255;y:255);
        LevelKennung:String[13]='Weltraumspiel';
  var K:Byte;
  begin
   BildDaten.Kennung:=LevelKennung;
   for K:=1 to 99 do BildDaten.Level[K]:=LeerRaum;
   LevelNr:=1;LevelErzeugen(LevelNr);
   SpinEditLevel.Value:=1;
   Eingabe.Repaint;
  end;

 procedure TLevelEditor.EingabeDrawCell(Sender: TObject; ACol,ARow: Integer;
                                        Rect: TRect; State: TGridDrawState);
  begin
   with Eingabe.Canvas do
    begin
     Brush.Style:=bsSolid;
     Brush.Color:=clBlack;
     Pen.Color:=clWhite;
     FillRect(Rect);
     if Level.Feld[ACol,ARow]=1 then
      begin
       Brush.Color:=clRed;
       Rectangle(Rect.Left+5,Rect.Top+5,Rect.Right-5,Rect.Bottom-5);
      end;
     if (Level.x=ACol) and (Level.y=ARow) then
      begin
       Brush.Color:=clGreen;
       Rectangle(Rect.Left+5,Rect.Top+5,Rect.Right-5,Rect.Bottom-5);
      end;
    end
  end;

 procedure TLevelEditor.SpinEditLevelChange(Sender: TObject);
  begin
   if SpinEditLevel.Text='' then SpinEditLevel.Value:=1;
   LevelSchreiben(LevelNr);
   LevelNr:=SpinEditLevel.Value;
   LevelErzeugen(LevelNr);Eingabe.Repaint;
   if Level.x>7 then SpinEditLevel.MaxValue:=LevelNr;
  end;

 procedure TLevelEditor.FormCreate(Sender: TObject);
  begin
   DateiNeuClick(Sender);
  end;

 procedure TLevelEditor.EingabeMouseDown(Sender: TObject; Button: TMouseButton;
                                         Shift: TShiftState; X, Y: Integer);
  var Col,Row:Integer;
  begin
   Eingabe.MouseToCell(X,Y,Col,Row);
   if Shift=[ssLeft] then
    begin
     Level.Feld[Col,Row]:=1-Level.Feld[Col,Row];
     if (Level.x=Col) and (Level.y=Row) then
      begin Level.x:=255;Level.y:=255 end
    end;
   if (Shift=[ssRight]) and (Level.Feld[Col,Row]=1) then
    begin Level.x:=Col;Level.y:=Row;SpinEditLevel.MaxValue:=99 end;
   Eingabe.Repaint;
  end;

 procedure TLevelEditor.LevelLoeschenClick(Sender: TObject);
  const LeerRaum:DatenRec=(Zeile:(0,0,0,0,0,0,0,0); x:255;y:255);
  begin
   BildDaten.Level[LevelNr]:=LeerRaum;
   LevelErzeugen(LevelNr);
   Eingabe.Repaint;
  end;

 procedure TLevelEditor.LevelZufallClick(Sender: TObject);
  var x,y,Anzahl:Integer;
  begin
   LevelLoeschenClick(Sender);
   x:=random(8);y:=random(8);LevelFertig:=False;
   Anzahl:=Random(20)+10;
   Level.x:=x;Level.y:=y;
   Level.Feld[x,y]:=1;
   Such(2,Anzahl,x,y);
   Eingabe.Repaint;
  end;

 procedure TLevelEditor.Such(Tiefe,Anzahl,x,y:Integer);
  const RichtungX:Array[1..8]of Integer=(-1,-2, 0, 0, 1, 2, 0, 0);
        RichtungY:Array[1..8]of Integer=( 0, 0,-1,-2, 0, 0, 1, 2);
  var N,K,Nr:Integer;
      ZielListe:Array[1..8]of TPoint;
      Benutzt:Array[1..8]of Boolean;
      Sprung:TPoint;
  begin
   if LevelFertig then exit;
   N:=0;
   for K:=1 to 8 do
    if Level.Feld[x+RichtungX[K],y+RichtungY[K]]=0 then
     begin
      inc(N);
      ZielListe[N].X:=x+RichtungX[K];
      ZielListe[N].Y:=y+RichtungY[K];
      Benutzt[N]:=False;
     end;
   for K:=1 to N do
    begin
     repeat
      Nr:=random(N)+1;
      Sprung:=ZielListe[Nr];
     until not Benutzt[Nr];
     Benutzt[Nr]:=True;
     Level.Feld[Sprung.X,Sprung.Y]:=1;
     if Tiefe<Anzahl then Such(Tiefe+1,Anzahl,Sprung.X,Sprung.Y)
     else LevelFertig:=True;
     if LevelFertig then exit
     else
      begin
       Level.Feld[Sprung.X,Sprung.Y]:=0;
       Benutzt[Nr]:=False;
      end
    end
  end;

 procedure TLevelEditor.BeendenClick(Sender: TObject);
  begin close end;

 procedure TLevelEditor.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  begin
   if MessageDlg('Beenden?',mtConfirmation,[mbYes,mbNo],0)=mrYes then
    begin
     if MessageDlg('LevelDatei speichern?',mtWarning,[mbYes,mbNo],0)=mrYes then
      SpeichernClick(Sender);
     CanClose:=True
    end
   else CanClose:=False
  end;

end.

